import React, { createContext, useContext, useMemo } from 'react';
import { useSupportedAssetList } from '@onomy/react-hub';
import { useAsyncStatuses } from '@onomy/react-utils';
import { OnomyHubConstants, DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useCreateOrderForm } from '../forms/useCreateOrderForm';
import { useMarketOrderForm } from '../forms/useMarketOrderForm';
// import { useOnexLiquidity } from "./forms/useOnexLiquidity";
import { useOnexDayTradeFrame } from '../TheGraph/useOnexDayTradeFrame';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';
import { useOnexMyActiveOrders, useOnexOrderHistory, useOnexPoolDetail } from '../hooks';
import { useIndexedOrderBookForm } from '../TheGraph/useOrderBookData';

function useExchangePairState({
  quoteAsset,
  baseAsset,
}: {
  quoteAsset: DetailedFungibleAssetOnChain;
  baseAsset: DetailedFungibleAssetOnChain;
}) {
  const createOrderForm = useCreateOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id,
  });
  const marketOrderForm = useMarketOrderForm({
    quoteDenom: quoteAsset.id,
    baseDenom: baseAsset.id,
    orderDirection: createOrderForm.direction,
  });
  const [dayTradeData, dayTradeDataStatus] = useOnexDayTradeFrame({ baseAsset, quoteAsset });
  const [history, historyStatus] = useOnexOrderHistory({ baseAsset, quoteAsset });
  const [pool, poolStatus] = useOnexPoolDetail({ baseAsset, quoteAsset });

  // const liquidity = useOnexLiquidity({ baseAsset, quoteAsset });

  const [orders, ordersStatus] = useOnexMyActiveOrders({ quoteAsset, baseAsset });

  const {
    binOptions: bookBinOptions,
    bookBuy,
    bookSell,
    selectedBin: bookSelectedBin,
    setSelectedBin: setBookSelectedBin,
  } = useIndexedOrderBookForm({
    baseAsset: createOrderForm.baseAsset,
    quoteAsset: createOrderForm.quoteAsset,
  });

  const loadingStatus = useAsyncStatuses([
    createOrderForm.loadingStatus,
    // dayTradeDataStatus,
    // dayTradeData.loadingStatus,
    historyStatus,
    poolStatus,
    // liquidity.loadingStatus,
    ordersStatus,
  ]);

  return {
    loadingStatus,
    dayTradeDataStatus,

    quoteAsset,
    baseAsset,
    currentPricePerBase: createOrderForm.currentPricePerBase,
    lastPricePerBase: dayTradeData.lastPricePerBase,

    dayTradeData,

    createOrderForm,
    marketOrderForm,

    history,
    // liquidity,
    pool,
    orders,

    bookBuy,
    bookSell,
    bookBinOptions,
    bookSelectedBin,

    orderDirection: createOrderForm.direction,

    setOrderDirection: createOrderForm.setDirection,
    setBookSelectedBin,
  };
}

export type ExchangePairContextType = ReturnType<typeof useExchangePairState>;

const ExchangePairContext = createContext<ExchangePairContextType>(null as any);

export function useExchangePair() {
  const state = useContext(ExchangePairContext);
  if (!state) throw new Error('ExchangePairContext missing');
  return state;
}

// eslint-disable-next-line react/function-component-definition
const DEFAULT_LOADING_RENDER_FN = () => (
  <PlaceholderLoadingIndicator text="Loading exchange pair" />
);
// eslint-disable-next-line react/function-component-definition
const DEFAULT_ERROR_RENDER_FN = (error: any) => <PlaceholderErrorDisplay error={error} />;

export function ExchangePairProviderInner({
  children,
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  quoteAsset: DetailedFungibleAssetOnChain;
  baseAsset: DetailedFungibleAssetOnChain;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useExchangePairState({ baseAsset, quoteAsset });

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <ExchangePairContext.Provider value={state}>{children}</ExchangePairContext.Provider>;
}

export function ExchangePairProvider({
  children,
  getBaseSymbol,
  getQuoteSymbol,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  getBaseSymbol: () => string;
  getQuoteSymbol: () => string;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const baseSymbol = getBaseSymbol();
  const quoteSymbol = getQuoteSymbol();
  const [allAssets, assetStatus] = useSupportedAssetList();
  const onexAssets = useMemo(
    () => allAssets.filter(x => x.chain.name === OnomyHubConstants.ONEX_CHAIN_ID),
    [allAssets]
  );

  const baseAsset = useMemo(
    () => onexAssets.find(x => x.meta.symbol.toLowerCase() === baseSymbol),
    [onexAssets, baseSymbol]
  );
  const quoteAsset = useMemo(
    () => onexAssets.find(x => x.meta.symbol.toLowerCase() === quoteSymbol),
    [onexAssets, quoteSymbol]
  );

  if (assetStatus.pending) return renderLoading();
  if (assetStatus.error) return renderError(assetStatus.error);
  if (!baseAsset || !quoteAsset) return <h2>Pair not found</h2>;

  return (
    <ExchangePairProviderInner
      baseAsset={baseAsset}
      quoteAsset={quoteAsset}
      renderError={renderError}
      renderLoading={renderLoading}
    >
      {children}
    </ExchangePairProviderInner>
  );
}
