import React, { createContext, useContext } from 'react';
import { DetailedFungibleAssetOnChain } from '@onomy/hub';
import { useOnexLiquidity } from '../forms/useOnexLiquidity';
import { PlaceholderLoadingIndicator } from '../../components/PlaceholderLoadingIndicator';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useLiquidityPairState({
  quoteAsset,
  baseAsset,
}: {
  quoteAsset: DetailedFungibleAssetOnChain;
  baseAsset: DetailedFungibleAssetOnChain;
}) {
  const liquidity = useOnexLiquidity({ baseAsset, quoteAsset });

  return {
    ...liquidity,
    baseAsset,
    quoteAsset,
  };
}

export type LiquidityPairContextType = ReturnType<typeof useLiquidityPairState>;

const LiquidityPairContext = createContext<LiquidityPairContextType>(null as any);

export function useLiquidityPair() {
  const state = useContext(LiquidityPairContext);
  if (!state) throw new Error('LiquidityPairContext missing');
  return state;
}

// eslint-disable-next-line react/function-component-definition
const DEFAULT_LOADING_RENDER_FN = () => (
  <PlaceholderLoadingIndicator text="Loading liquidity pair" />
);
// eslint-disable-next-line react/function-component-definition
const DEFAULT_ERROR_RENDER_FN = (error: any) => <PlaceholderErrorDisplay error={error} />;

export function LiquidityPairProvider({
  quoteAsset,
  baseAsset,
  renderLoading = DEFAULT_LOADING_RENDER_FN,
  renderError = DEFAULT_ERROR_RENDER_FN,
  children,
}: {
  children: JSX.Element | JSX.Element[];
  quoteAsset: DetailedFungibleAssetOnChain;
  baseAsset: DetailedFungibleAssetOnChain;
  renderLoading?: typeof DEFAULT_LOADING_RENDER_FN;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useLiquidityPairState({ baseAsset, quoteAsset });

  if (state.loadingStatus.pending) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);

  return <LiquidityPairContext.Provider value={state}>{children}</LiquidityPairContext.Provider>;
}
