import { CosmosClient, OnomyClient } from '@onomy/client';
import { FungibleAssetBalance } from '@onomy/frontend-relay-types';
import { DetailedFungibleAssetBalance, OnomyHub, OnomyHubUtils } from '@onomy/hub';
import { serializeAsset, useOnomyHub } from '@onomy/react-hub';
import { useAsyncPoll } from '@onomy/react-utils';
import { FungibleAssetOnChain } from 'index';
import { useCallback } from 'react';

async function includeVestingDetails({
  client,
  hub,
  vesting,
}: {
  client: CosmosClient;
  hub: ReturnType<typeof useOnomyHub>['hub'];
  vesting: Awaited<ReturnType<OnomyClient['getVesting']>>;
}) {
  if (!hub) throw new Error('Hub is null');
  if (!vesting) return null;

  const now = new Date().getTime() / 1000;
  const startTime = vesting.startTime.toNumber();

  const chain = client.getBlockchain();

  function coinDetails(coin: { denom: string; amount: string }) {
    return hub!.balances.includeDetails({
      amount: BigInt(coin.amount),
      asset: {
        id: coin.denom,
        chain,
      },
    });
  }

  async function getBaseVestingAccount() {
    const [delegatedFree, delegatedVesting, originalVesting] = await Promise.all([
      Promise.all(vesting!.baseVestingAccount.delegatedFree.map(coinDetails)),
      Promise.all(vesting!.baseVestingAccount.delegatedVesting.map(coinDetails)),
      Promise.all(vesting!.baseVestingAccount.originalVesting.map(coinDetails)),
    ]);
    return {
      ...vesting!,
      delegatedFree,
      delegatedVesting,
      originalVesting,
    };
  }

  async function getVestingPeriods() {
    let totalWait = 0;

    const periods =
      vesting?.vestingPeriods.map(period => {
        totalWait += period.length.toNumber();
        const unlocksAt = startTime + totalWait;
        const isUnlocked = now >= unlocksAt;
        return {
          ...period,
          totalWait,
          unlocksAt,
          isUnlocked,
        };
      }) ?? [];

    return Promise.all(
      periods.map(async period => {
        const amounts = await Promise.all(period.amount.map(coinDetails));
        return {
          ...period,
          amounts,
        };
      }) ?? []
    );
  }

  const [baseVestingAccount, vestingPeriods] = await Promise.all([
    getBaseVestingAccount(),
    getVestingPeriods(),
  ]);

  const allLockedRaw = vestingPeriods.reduce((res, period) => {
    if (!period.isUnlocked) return res;
    return [...res, ...period.amounts];
  }, [] as DetailedFungibleAssetBalance[]);

  const allLockedCombined = OnomyHub.Utils.combineBalances(allLockedRaw);
  const allLocked = await Promise.all(
    allLockedCombined.map(amount => hub.balances.includeDetails(amount))
  );

  const zero = BigInt(0);

  const allLockedStakedRaw = [] as FungibleAssetBalance[];
  const allLockedUnstakedRaw = allLocked.map(amt => {
    const assetId = serializeAsset(amt.asset);
    const delegatedVesting =
      baseVestingAccount.delegatedVesting.find(a => serializeAsset(a.asset) === assetId)?.amount ??
      zero;
    const delegatedFree =
      baseVestingAccount.delegatedFree.find(a => serializeAsset(a.asset) === assetId)?.amount ??
      zero;

    let staked = delegatedVesting - delegatedFree;
    if (staked < zero) staked = zero;
    let amount = amt.amount - staked;
    if (amount < zero) amount = zero;

    let stakedAmount = amt.amount - amount;
    if (stakedAmount < zero) stakedAmount = zero;

    allLockedStakedRaw.push({
      asset: amt.asset,
      amount: stakedAmount,
    });

    return {
      asset: amt.asset,
      amount,
    };
  });

  const [allLockedStaked, allLockedUnstaked] = await Promise.all([
    Promise.all(allLockedStakedRaw.map(amount => hub.balances.includeDetails(amount))),
    Promise.all(allLockedUnstakedRaw.map(amount => hub.balances.includeDetails(amount))),
  ]);

  const allLockedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
  const allLockedStakedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
  const allLockedUnstakedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);

  return {
    ...vesting,
    chain,
    baseVestingAccount,
    vestingPeriods,
    allLocked,
    allLockedStaked,
    allLockedUnstaked,
    allLockedFiatValue,
    allLockedStakedFiatValue,
    allLockedUnstakedFiatValue,
  };
}

export function useVesting() {
  const { hub } = useOnomyHub();
  const cb = useCallback(async () => {
    if (!hub) return null;
    const clients: CosmosClient[] = [hub.chains.onomy.client];
    if (hub.chains.onex) clients.push(hub.chains.onex.client);

    const vestingDataRaw = await Promise.all(
      clients.map(async client => {
        return includeVestingDetails({
          client,
          hub,
          vesting: await (async () => {
            try {
              return await client.getVesting();
            } catch (e: any) {
              if (e?.message?.match(/not found/)) return null;
              throw e;
            }
          })(),
        });
      })
    );

    const vestingData = vestingDataRaw.filter((x): x is Exclude<typeof x, null> => x !== null);

    if (!vestingData.length) return null;
    const allLockedRaw = vestingData.reduce((res, item) => {
      return [...res, ...(item?.allLocked ?? [])];
    }, [] as DetailedFungibleAssetBalance[]);
    const allLockedStakedRaw = vestingData.reduce((res, item) => {
      return [...res, ...(item?.allLockedStaked ?? [])];
    }, [] as DetailedFungibleAssetBalance[]);
    const allLockedUnstakedRaw = vestingData.reduce((res, item) => {
      return [...res, ...(item?.allLockedUnstaked ?? [])];
    }, [] as DetailedFungibleAssetBalance[]);
    const allLockedCombined = OnomyHubUtils.combineBalances(allLockedRaw);
    const allLockedStakedCombined = OnomyHubUtils.combineBalances(allLockedStakedRaw);
    const allLockedUnstakedCombined = OnomyHubUtils.combineBalances(allLockedUnstakedRaw);

    const [allLocked, allLockedStaked, allLockedUnstaked] = await Promise.all([
      Promise.all(allLockedCombined.map(amount => hub.balances.includeDetails(amount))),
      Promise.all(allLockedStakedCombined.map(amount => hub.balances.includeDetails(amount))),
      Promise.all(allLockedUnstakedCombined.map(amount => hub.balances.includeDetails(amount))),
    ]);

    const allLockedFiatValue = allLocked.reduce((res, item) => res + item.fiatValue, 0);
    const allLockedStakedFiatValue = allLockedStaked.reduce((res, item) => res + item.fiatValue, 0);
    const allLockedUnstakedFiatValue = allLockedUnstaked.reduce(
      (res, item) => res + item.fiatValue,
      0
    );

    return {
      allLocked,
      allLockedStaked,
      allLockedUnstaked,
      allLockedFiatValue,
      allLockedStakedFiatValue,
      allLockedUnstakedFiatValue,
      vestingData,
    };
  }, [hub]);

  return useAsyncPoll(cb, null);
}
