import React, { useCallback, useMemo, createContext, useContext } from 'react';
import { ApolloClient, InMemoryCache } from '@apollo/client';
import { useAsyncStatuses } from '@onomy/react-utils';
import { DetailedFungibleAssetOnChain, OnomyHubConstants } from '@onomy/hub';
import { serializeAsset, useAddress, useOnomyHub, useSupportedAssetList } from '@onomy/react-hub';
import { FungibleAssetOnChain } from '@onomy/frontend-relay-types';

import { OnomyEcosystemConfig } from '../../OnomyEcosystemConfigs';
import { useExchangePairs } from '../../exchange/forms/useExchangePairs';
import { OnexHistoryFrames } from '../../exchange/TheGraph/OnexHistoryFrames';
import { PlaceholderErrorDisplay } from '../../components/PlaceholderErrorDisplay';

function useFindAsset(assetId: string, supportedAssets: DetailedFungibleAssetOnChain[]) {
  return React.useMemo(
    () => supportedAssets.find(a => serializeAsset(a) === assetId) ?? null,
    [supportedAssets, assetId]
  );
}

function useOnomyAppState(ecosystem: OnomyEcosystemConfig) {
  const { wallet, hub } = useOnomyHub();
  const [supportedAssets, assetsStatus] = useSupportedAssetList();
  const supportedChains = useMemo(
    () => [
      OnomyHubConstants.ONOMY_CHAIN,
      ...(ecosystem.onexChainInfo ? [OnomyHubConstants.ONEX_CHAIN] : []),
      ...(ecosystem.ethWsUrl ? [OnomyHubConstants.ETH_CHAIN] : []),
    ],
    [ecosystem.onexChainInfo, ecosystem.ethWsUrl]
  );
  const [myOnomyAddress, onomyAddressStatus] = useAddress(OnomyHubConstants.ONOMY_CHAIN);
  const [myOnexAddress, onexAddressStatus] = useAddress(OnomyHubConstants.ONEX_CHAIN);

  const onomyNomAsset =
    useFindAsset(`${OnomyHubConstants.ONOMY_CHAIN.name}_anom`, supportedAssets) ??
    OnomyHubConstants.BNOM_ASSET;
  const onexOnexAsset =
    useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_aonex`, supportedAssets) ??
    OnomyHubConstants.ONEX_ASSET;
  const onexNomAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_anom`, supportedAssets);
  const onexUsdtAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_ausdt`, supportedAssets);
  const onexUsdcAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_ausdc`, supportedAssets);
  const onexEthAsset = useFindAsset(`${OnomyHubConstants.ONEX_CHAIN.name}_wei`, supportedAssets);

  const { loadingStatus: pairsStatus, quoteAssets, pairsByQuote } = useExchangePairs();

  const isOnexAvailable = !!ecosystem.onexChainInfo;

  const onexHistoryFrames = useMemo(() => {
    if (!ecosystem.onexGraphqlEndpoint) return null;
    const apolloClient = new ApolloClient({
      uri: ecosystem.onexGraphqlEndpoint,
      cache: new InMemoryCache(),
    });

    return new OnexHistoryFrames(apolloClient);
  }, [ecosystem.onexGraphqlEndpoint]);

  const myOnomyWallet = wallet.onomy.connectedWallet;
  const myOnexWallet = wallet.onex.connectedWallet;

  const loadingStatus = useAsyncStatuses([
    assetsStatus,
    pairsStatus,
    onomyAddressStatus,
    onexAddressStatus,
  ]);

  const logout = useCallback(async () => {
    await wallet.disconnect();
    window?.location?.reload();
  }, [wallet]);

  const logoutOnomy = useCallback(async () => {
    await wallet.onomy.disconnect();
    window.location?.reload();
  }, [wallet.onomy]);

  const logoutOnex = useCallback(async () => {
    await wallet.onex.disconnect();
    window.location?.reload();
  }, [wallet.onex]);

  return {
    ecosystem,
    isOnexAvailable,

    loadingStatus,

    bridges: hub?.bridges ?? [],

    onexHistoryFrames,

    onomyChain: OnomyHubConstants.ONOMY_CHAIN,
    onexChain: OnomyHubConstants.ONEX_CHAIN,

    supportedChains,
    supportedAssets,

    onomyNomAsset,
    onexOnexAsset,
    onexNomAsset,
    onexUsdtAsset,
    onexUsdcAsset,
    onexEthAsset,

    unknownBalance: OnomyHubConstants.UNKNOWN_BALANCE,

    myOnomyAddress,
    myOnexAddress,
    myOnomyWallet,
    myOnexWallet,

    quoteAssets,
    pairsByQuote,

    logout,
    logoutOnomy,
    logoutOnex,

    isWalletLoaded: wallet.isLoaded,
    walletLoadError: wallet.error,
  };
}

export type OnomyAppContextType = ReturnType<typeof useOnomyAppState>;

const OnomyAppContext = createContext<OnomyAppContextType>(null as any);

export function useOnomyApp() {
  const state = useContext(OnomyAppContext);
  if (!state) throw new Error('OnomyAppContext missing');
  return state;
}

// eslint-disable-next-line @typescript-eslint/naming-convention
function DEFAULT_ERROR_RENDER_FN(error: any) {
  return <PlaceholderErrorDisplay error={error} />;
}

export function OnomyAppProvider({
  children,
  ecosystem,
  renderLoading,
  renderError = DEFAULT_ERROR_RENDER_FN,
}: {
  children: JSX.Element | JSX.Element[];
  ecosystem: OnomyEcosystemConfig;
  renderLoading?: () => JSX.Element | null;
  renderError?: typeof DEFAULT_ERROR_RENDER_FN;
}) {
  const state = useOnomyAppState(ecosystem);

  if (state.loadingStatus.pending && renderLoading) return renderLoading();
  if (state.loadingStatus.error) return renderError(state.loadingStatus.error);
  return <OnomyAppContext.Provider value={state}>{children}</OnomyAppContext.Provider>;
}

export function useAsset(idOrAsset: string | FungibleAssetOnChain) {
  const { supportedAssets } = useOnomyApp();
  const id = typeof idOrAsset === 'string' ? idOrAsset : serializeAsset(idOrAsset);
  return useFindAsset(id, supportedAssets);
}
